@@ -2,17 +2,17 @@ from __future__ import with_statement
 import errno
 import os
 import psutil
+import select
 import signal
-import socket
 import sys
 import time
 import threading
 
 try:  # Python 2.6 unittest module doesn't have skip decorators.
-    from unittest import skip, skipIf, skipUnless
+    from unittest import skipIf, skipUnless
     import unittest
 except ImportError:
-    from unittest2 import skip, skipIf, skipUnless
+    from unittest2 import skipIf, skipUnless
     import unittest2 as unittest
 
 try:  # Python 2.x doesn't define time.perf_counter.
@@ -43,10 +43,38 @@ SHORT_SELECT = 0.01
 # Tolerance values for timer/speed fluctuations.
 TOLERANCE = 0.75
 
-# Detect whether we're running on Travis.  This is used
-# to skip some verification points inside of tests to
+# Detect whether we're running on Travis or AppVeyor.  This
+# is used to skip some verification points inside of tests to
 # not randomly fail our CI due to wild timer/speed differences.
 TRAVIS_CI = "TRAVIS" in os.environ
+APPVEYOR = "APPVEYOR" in os.environ
+
+
+skipUnlessHasSelector = skipUnless(selectors.HAS_SELECT, "Platform doesn't have a selector")
+skipUnlessHasENOSYS = skipUnless(hasattr(errno, 'ENOSYS'), "Platform doesn't have errno.ENOSYS")
+skipUnlessHasAlarm = skipUnless(hasattr(signal, 'alarm'), "Platform doesn't have signal.alarm()")
+
+
+def patch_select_module(testcase, *keep, **replace):
+    """ Helper function that removes all selectors from the select module
+    except those listed in *keep and **replace. Those in keep will be kept
+    if they exist in the select module and those in replace will be patched
+    with the value that is given regardless if they exist or not. Cleanup
+    will restore previous state. This helper also resets the selectors module
+    so that a call to DefaultSelector() will do feature detection again. """
+    selectors._DEFAULT_SELECTOR = None
+    for s in ['select', 'poll', 'epoll', 'kqueue']:
+        if s in replace:
+            if hasattr(select, s):
+                old_selector = getattr(select, s)
+                testcase.addCleanup(setattr, select, s, old_selector)
+            else:
+                testcase.addCleanup(delattr, select, s)
+            setattr(select, s, replace[s])
+        elif s not in keep and hasattr(select, s):
+            old_selector = getattr(select, s)
+            testcase.addCleanup(setattr, select, s, old_selector)
+            delattr(select, s)
 
 
 class AlarmThread(threading.Thread):
@@ -69,8 +97,6 @@ class AlarmMixin(object):
     alarm_thread = None
 
     def _begin_alarm_thread(self, timeout):
-        if not HAS_ALARM:
-            self.skipTest("Platform doesn't have signal.SIGALRM")
         self.addCleanup(self._cancel_alarm_thread)
         self.alarm_thread = AlarmThread(timeout)
         self.alarm_thread.start()
@@ -103,7 +129,7 @@ class TimerContext(object):
         total_time = self.end_time - self.start_time
 
         # Skip timing on CI due to flakiness.
-        if TRAVIS_CI:
+        if TRAVIS_CI or APPVEYOR:
             return
 
         if self.lower is not None:
@@ -117,13 +143,12 @@ class TimerMixin(object):
         return TimerContext(self, lower=lower, upper=upper)
 
 
-@skipUnless(selectors.HAS_SELECT, "Platform doesn't have a selector")
+@skipUnlessHasSelector
 class BaseSelectorTestCase(unittest.TestCase, AlarmMixin, TimerMixin):
     """ Implements the tests that each type of selector must pass. """
-    SELECTOR = selectors.DefaultSelector
 
     def make_socketpair(self):
-        rd, wr = socket.socketpair()
+        rd, wr = socketpair()
 
         # Make non-blocking so we get errors if the
         # sockets are interacted with but not ready.
@@ -135,7 +160,7 @@ class BaseSelectorTestCase(unittest.TestCase, AlarmMixin, TimerMixin):
         return rd, wr
 
     def make_selector(self):
-        s = self.SELECTOR()
+        s = selectors.DefaultSelector()
         self.addCleanup(s.close)
         return s
 
@@ -418,7 +443,7 @@ class BaseSelectorTestCase(unittest.TestCase, AlarmMixin, TimerMixin):
         with self.assertTakesTime(lower=SHORT_SELECT, upper=SHORT_SELECT):
             self.assertEqual(0, len(s.select(timeout=SHORT_SELECT)))
 
-    @skipUnless(HAS_ALARM, "Platform doesn't have signal.alarm()")
+    @skipUnlessHasAlarm
     def test_select_timing(self):
         s = self.make_selector()
         rd, wr = self.make_socketpair()
@@ -430,7 +455,7 @@ class BaseSelectorTestCase(unittest.TestCase, AlarmMixin, TimerMixin):
             ready = s.select(LONG_SELECT)
         self.assertEqual([(key, selectors.EVENT_READ)], ready)
 
-    @skipUnless(HAS_ALARM, "Platform doesn't have signal.alarm()")
+    @skipUnlessHasAlarm
     def test_select_interrupt_no_event(self):
         s = self.make_selector()
         rd, wr = self.make_socketpair()
@@ -441,7 +466,7 @@ class BaseSelectorTestCase(unittest.TestCase, AlarmMixin, TimerMixin):
         with self.assertTakesTime(lower=LONG_SELECT, upper=LONG_SELECT):
             self.assertEqual([], s.select(LONG_SELECT))
 
-    @skipUnless(HAS_ALARM, "Platform doesn't have signal.alarm()")
+    @skipUnlessHasAlarm
     def test_select_interrupt_with_event(self):
         s = self.make_selector()
         rd, wr = self.make_socketpair()
@@ -454,7 +479,7 @@ class BaseSelectorTestCase(unittest.TestCase, AlarmMixin, TimerMixin):
             self.assertEqual([(key, selectors.EVENT_READ)], s.select(LONG_SELECT))
         self.assertEqual(rd.recv(1), b'x')
 
-    @skipUnless(HAS_ALARM, "Platform doesn't have signal.alarm()")
+    @skipUnlessHasAlarm
     def test_select_multiple_interrupts_with_event(self):
         s = self.make_selector()
         rd, wr = self.make_socketpair()
@@ -474,7 +499,7 @@ class BaseSelectorTestCase(unittest.TestCase, AlarmMixin, TimerMixin):
             self.assertEqual([(key, selectors.EVENT_READ)], s.select(LONG_SELECT))
         self.assertEqual(rd.recv(1), b'x')
 
-    @skipUnless(HAS_ALARM, "Platform doesn't have signal.alarm()")
+    @skipUnlessHasAlarm
     def test_selector_error(self):
         s = self.make_selector()
         rd, wr = self.make_socketpair()
@@ -498,7 +523,7 @@ class BaseSelectorTestCase(unittest.TestCase, AlarmMixin, TimerMixin):
 
     # Test ensures that _syscall_wrapper properly raises the
     # exception that is raised from an interrupt handler.
-    @skipUnless(HAS_ALARM, "Platform doesn't have signal.alarm()")
+    @skipUnlessHasAlarm
     def test_select_interrupt_exception(self):
         s = self.make_selector()
         rd, wr = self.make_socketpair()
@@ -536,22 +561,15 @@ class BaseSelectorTestCase(unittest.TestCase, AlarmMixin, TimerMixin):
         after_fds = len(proc.open_files())
         self.assertEqual(before_fds, after_fds)
 
-    def test_selector_error(self):
+    def test_selector_error_exception(self):
         err = selectors.SelectorError(1)
         self.assertEqual(err.__repr__(), "<SelectorError errno=1>")
         self.assertEqual(err.__str__(), "<SelectorError errno=1>")
 
 
 class BaseWaitForTestCase(unittest.TestCase, TimerMixin, AlarmMixin):
-    SELECTOR = selectors.DefaultSelector
-
-    def setUp(self):
-        old_selector = wait.DefaultSelector
-        wait.DefaultSelector = self.SELECTOR
-        self.addCleanup(setattr, wait, "DefaultSelector", old_selector)
-
     def make_socketpair(self):
-        rd, wr = socket.socketpair()
+        rd, wr = socketpair()
 
         # Make non-blocking so we get errors if the
         # sockets are interacted with but not ready.
@@ -562,11 +580,6 @@ class BaseWaitForTestCase(unittest.TestCase, TimerMixin, AlarmMixin):
         self.addCleanup(wr.close)
         return rd, wr
 
-    def make_selector(self):
-        s = self.SELECTOR()
-        self.addCleanup(s.close)
-        return s
-
     def test_wait_for_read_single_socket(self):
         rd, wr = self.make_socketpair()
         self.assertEqual([], wait.wait_for_read(rd, timeout=SHORT_SELECT))
@@ -605,10 +618,10 @@ class BaseWaitForTestCase(unittest.TestCase, TimerMixin, AlarmMixin):
             wait.wait_for_read([rd], timeout=SHORT_SELECT)
 
     def test_wait_io_close_is_called(self):
-        selector = self.SELECTOR()
+        selector = selectors.DefaultSelector()
         self.addCleanup(selector.close)
 
-        def fake_constructor(*args, **kwargs):
+        def fake_constructor():
             return selector
 
         old_selector = wait.DefaultSelector
@@ -619,7 +632,7 @@ class BaseWaitForTestCase(unittest.TestCase, TimerMixin, AlarmMixin):
         wait.wait_for_write([rd, wr], 0.001)
         self.assertIs(selector._map, None)
 
-    @skipUnless(HAS_ALARM, "Platform doesn't have signal.alarm()")
+    @skipUnlessHasAlarm
     def test_interrupt_wait_for_read_no_event(self):
         rd, wr = self.make_socketpair()
 
@@ -627,7 +640,7 @@ class BaseWaitForTestCase(unittest.TestCase, TimerMixin, AlarmMixin):
         with self.assertTakesTime(lower=LONG_SELECT, upper=LONG_SELECT):
             self.assertEqual([], wait.wait_for_read(rd, timeout=LONG_SELECT))
 
-    @skipUnless(HAS_ALARM, "Platform doesn't have signal.alarm()")
+    @skipUnlessHasAlarm
     def test_interrupt_wait_for_read_with_event(self):
         rd, wr = self.make_socketpair()
 
@@ -681,41 +694,102 @@ class ScalableSelectorMixin(object):
         self.assertEqual(limit_nofile // 2, len(s.select()))
 
 
+@skipUnlessHasSelector
+class TestUniqueSelectScenarios(BaseSelectorTestCase):
+    def test_select_module_patched_after_import(self):
+        # This test is to make sure that after import time
+        # calling DefaultSelector() will still give a good
+        # return value. This issue is caused by gevent, eventlet.
+
+        # Now remove all selectors except `select.select`.
+        patch_select_module(self, 'select')
+
+        # Make sure that the selector returned only uses the selector available.
+        selector = self.make_selector()
+        self.assertIsInstance(selector, selectors.SelectSelector)
+
+    @skipUnlessHasENOSYS
+    def test_select_module_defines_does_not_implement_poll(self):
+        # This test is to make sure that if a platform defines
+        # a selector as being available but does not actually
+        # implement it (kennethreitz/requests#3906) then
+        # DefaultSelector() does not fail.
+
+        # Reset the _DEFAULT_SELECTOR value as if using for the first time.
+        selectors._DEFAULT_SELECTOR = None
+
+        # Now we're going to patch in a bad `poll`.
+        class BadPoll(object):
+            def poll(self, timeout):
+                raise OSError(errno.ENOSYS)
+
+        # Remove all selectors except `select.select` and replace `select.poll`.
+        patch_select_module(self, 'select', poll=BadPoll)
+
+        selector = self.make_selector()
+        self.assertIsInstance(selector, selectors.SelectSelector)
+
+    @skipUnlessHasENOSYS
+    def test_select_module_defines_does_not_implement_epoll(self):
+        # Same as above test except with `select.epoll`.
+
+        # Reset the _DEFAULT_SELECTOR value as if using for the first time.
+        selectors._DEFAULT_SELECTOR = None
+
+        # Now we're going to patch in a bad `epoll`.
+        def bad_epoll(*args, **kwargs):
+            raise OSError(errno.ENOSYS)
+
+        # Remove all selectors except `select.select` and replace `select.epoll`.
+        patch_select_module(self, 'select', epoll=bad_epoll)
+
+        selector = self.make_selector()
+        self.assertIsInstance(selector, selectors.SelectSelector)
+
+
 @skipUnless(hasattr(selectors, "SelectSelector"), "Platform doesn't have a SelectSelector")
 class SelectSelectorTestCase(BaseSelectorTestCase):
-    SELECTOR = getattr(selectors, "SelectSelector", None)
+    def setUp(self):
+        patch_select_module(self, 'select')
 
 
 @skipUnless(hasattr(selectors, "PollSelector"), "Platform doesn't have a PollSelector")
 class PollSelectorTestCase(BaseSelectorTestCase, ScalableSelectorMixin):
-    SELECTOR = getattr(selectors, "PollSelector", None)
+    def setUp(self):
+        patch_select_module(self, 'poll')
 
 
 @skipUnless(hasattr(selectors, "EpollSelector"), "Platform doesn't have an EpollSelector")
 class EpollSelectorTestCase(BaseSelectorTestCase, ScalableSelectorMixin):
-    SELECTOR = getattr(selectors, "EpollSelector", None)
+    def setUp(self):
+        patch_select_module(self, 'epoll')
 
 
 @skipUnless(hasattr(selectors, "KqueueSelector"), "Platform doesn't have a KqueueSelector")
 class KqueueSelectorTestCase(BaseSelectorTestCase, ScalableSelectorMixin):
-    SELECTOR = getattr(selectors, "KqueueSelector", None)
+    def setUp(self):
+        patch_select_module(self, 'kqueue')
 
 
 @skipUnless(hasattr(selectors, "SelectSelector"), "Platform doesn't have a SelectSelector")
 class SelectWaitForTestCase(BaseWaitForTestCase):
-    SELECTOR = getattr(selectors, "SelectSelector", None)
+    def setUp(self):
+        patch_select_module(self, 'select')
 
 
 @skipUnless(hasattr(selectors, "PollSelector"), "Platform doesn't have a PollSelector")
 class PollWaitForTestCase(BaseWaitForTestCase):
-    SELECTOR = getattr(selectors, "PollSelector", None)
+    def setUp(self):
+        patch_select_module(self, 'poll')
 
 
 @skipUnless(hasattr(selectors, "EpollSelector"), "Platform doesn't have an EpollSelector")
 class EpollWaitForTestCase(BaseWaitForTestCase):
-    SELECTOR = getattr(selectors, "EpollSelector", None)
+    def setUp(self):
+        patch_select_module(self, 'epoll')
 
 
 @skipUnless(hasattr(selectors, "KqueueSelector"), "Platform doesn't have a KqueueSelector")
 class KqueueWaitForTestCase(BaseWaitForTestCase):
-    SELECTOR = getattr(selectors, "KqueueSelector", None)
+    def setUp(self):
+        patch_select_module(self, 'kqueue')
